
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 * Implementation historique.
 *
 */


//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z


#include "historique_partie.h"


#include <assert.h>
#include <stdlib.h>
#include <stdio.h>


void deplacement_invalider(deplacement_piece* deplacement)
{
    assert(deplacement!=NULL);

    //ensemble des champs de coordonnees a -1
    deplacement->x_depart=-1;
    deplacement->y_depart=-1;
    deplacement->x_arrivee=-1;
    deplacement->y_arrivee=-1;

    //type de deplacement
    deplacement->type=deplacement_invalide;
}

int deplacement_est_valide(const deplacement_piece* deplacement)
{
    assert(deplacement!=NULL);

    //deplacement est valide si les coordoonnees sont valides, et le type de deplacement est valide.
    return (coordonnee_est_valide(deplacement->x_depart,deplacement->y_depart)==1 &&
            coordonnee_est_valide(deplacement->x_arrivee,deplacement->y_arrivee)==1 &&
            type_deplacement_est_valide(deplacement->type)==1);

}




void historique_initialise(historique_partie* historique)
{
    assert(historique!=NULL);

    //place l'ensemble des deplacement a un etat invalide
    int k=0;
    for(k=0;k<MAX_HISTORIQUE;++k)
        deplacement_invalider(&historique->tableau_coup[k]);
    historique->nombre_de_coup=0;

    //initialise l'echiquier
    echiquier echiquier_initial;
    echiquier_initialiser(&echiquier_initial);
    historique->echiquier_initial=echiquier_initial;
}

int historique_est_valide(const historique_partie* historique)
{
    assert(historique!=NULL);

    //Algorithme:
    //Verifie si nombre de coup est dans [0,MAX_HISTORIQUE]
    //Verifie que tout deplacement <nombre_de_coup est valide
    //Verifie que tout deplacement >=nombre_de_coup est invalide.

    int nbr=historique->nombre_de_coup;
    if(nbr<0 || nbr>=MAX_HISTORIQUE)
        return 0;
    int k=0;
    for(k=0;k<nbr;++k)
    {
        if(deplacement_est_valide(&historique->tableau_coup[k])!=1)
            return 0;
    }
    for(k=nbr;k<MAX_HISTORIQUE;++k)
        if(deplacement_est_valide(&historique->tableau_coup[k])==1)
            return 0;

    return 1;
}



void historique_ajoute_coup(historique_partie* historique,int x_depart,int y_depart,int x_arrivee,int y_arrivee,type_deplacement coup)
{
    //verification preconditions

    assert(historique!=NULL);
    assert(historique_est_valide(historique));
    assert(coordonnee_est_valide(x_depart,y_depart)==1);
    assert(coordonnee_est_valide(x_arrivee,y_arrivee)==1);
    assert(type_deplacement_est_valide(coup)==1);

    assert(historique->nombre_de_coup<MAX_HISTORIQUE-1);

    //protection utilisateur (erreur utilisateur possible)
    if(historique->nombre_de_coup>=MAX_HISTORIQUE-1)
    {
        printf("Erreur trop de coups joues, fin de partie\n");
        abort();
    }


    //recuperation deplacement courant
    deplacement_piece* deplacement=&historique->tableau_coup[historique->nombre_de_coup];
    assert(deplacement!=NULL);

    //mise a jour du deplacement
    deplacement->x_depart=x_depart;
    deplacement->y_depart=y_depart;
    deplacement->x_arrivee=x_arrivee;
    deplacement->y_arrivee=y_arrivee;
    deplacement->type=coup;

    //verification validitee post-condition sur le deplacement
    assert(deplacement_est_valide(deplacement)==1);

    //mise a jour du nombre de coups
    historique->nombre_de_coup++;

    //verification post-condition: validite de l'historique
    assert(historique_est_valide(historique)==1);
}


const deplacement_piece* historique_recupere_dernier_deplacement_information(const historique_partie* historique)
{
    assert(historique!=NULL);
    assert(historique_est_valide(historique)==1);

    if(historique->nombre_de_coup>0)
        return &historique->tableau_coup[historique->nombre_de_coup-1];

    //renvoie NULL dans le cas ou il s'agit du premier coup
    return NULL;

}


void historique_affiche_debug(const historique_partie* historique)
{
    assert(historique!=NULL);


    int N=historique->nombre_de_coup;
    int k=0;
    printf("Etat initial echiquier\n");
    echiquier_afficher_debug(&historique->echiquier_initial);
    printf("historique: %d coups\n",N);
    for(k=0;k<N;++k)
    {
        const deplacement_piece* d=&historique->tableau_coup[k];
        printf("[%d] %d %d -> %d %d\n",k,d->x_depart,d->y_depart,d->x_arrivee,d->y_arrivee);
    }
}







