

/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 * Implementation du parseur.
 *
 */

//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z

#include "parseur_mode_interactif.h"


#include <string.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

void action_initialise(action_a_realiser* action_a_initialiser)
{

    assert(action_a_initialiser!=NULL);

    // Type d'action vaut "rien_a_faire"
    // Nom de fichier vaut "\0"
    // Coordonnees vallent -1


    action_a_initialiser->type=rien_a_faire;
    strcpy(action_a_initialiser->filename,"\0");
    action_a_initialiser->x_depart=-1;
    action_a_initialiser->y_depart=-1;
    action_a_initialiser->x_arrivee=-1;
    action_a_initialiser->y_arrivee=-1;
}

void parse_ligne_commande(const char* ligne_commande,action_a_realiser* action_demandee)
{
    assert(action_demandee!=NULL);

    // Initialise action
    // Si ligne de commande commence par '#' => ne fait rien (commentaires)
    // Si ligne de commande vaut "fin" ou "FIN" => demande action de type "fin"
    // Si ligne de commande commence par 'i' => initialise l'echiquier
    // Si ligne de commande commence par 'd'
    //    Si la syntaxe correspond a "d x0 y0 -> x1 y1"
    //        Alors demande action deplacement de (x0,y0) vers (x1,y1)
    // Si ligne de commande commence par "lit"
    //    Si syntaxe correspond a "lit NOM_FICHIER"
    //        Alors demande action lecture fichier NOM_FICHIER
    // Si ligne de commande commence par "ecrit"
    //    Si syntaxe correspond a "ecrit NOM_FICHIER"
    //        Alors demande action ecriture fichier NOM_FICHIER
    // Si aucune reconnaissance, affiche que la ligne est invalide


    action_initialise(action_demandee);

    //protection en mode release
    if(ligne_commande==NULL)
        return ;

    //commentaires
    if(strlen(ligne_commande)>=1 && ligne_commande[0]=='#')
        return ;

    //fin du jeu
    else if(strlen(ligne_commande)>=3 &&
            (strncmp(ligne_commande,"FIN",3)==0 || strncmp(ligne_commande,"fin",3)==0) )
    {
        action_demandee->type=fin;
        return ;
    }

    //demande de deplacement
    else if(strlen(ligne_commande)>=2 && ligne_commande[0]=='d' && ligne_commande[1]==' ')
    {
        int x0=0;int y0=0;int x1=0;int y1=0;
        if(sscanf(ligne_commande,"d %d %d -> %d %d",&x0,&y0,&x1,&y1)==4)
        {
            //printf("[debug]: parsing deplacement %d,%d => %d,%d\n",x0,y0,x1,y1);fflush(stdout);

            action_demandee->type=deplacement;
            action_demandee->x_depart=x0;
            action_demandee->y_depart=y0;
            action_demandee->x_arrivee=x1;
            action_demandee->y_arrivee=y1;

            return ;
        }
    }

    //initialisation (uniquement verificiation si la premiere lettre vaut 'i')
    else if(strlen(ligne_commande)>=1 && ligne_commande[0]=='i')
    {
        action_demandee->type=initialisation;
        return;
    }

    //lecture dans un fichier
    else if(strlen(ligne_commande)>=4 && strncmp(ligne_commande,"lit ",4)==0)
    {
        char buffer_filename[MAX_NOM_FICHIER_TAILLE];
        if(sscanf(ligne_commande,"lit %s",buffer_filename)==1)
        {
            printf("[debug Parseur]: parsing chargement %s\n",buffer_filename);fflush(stdout);

            action_demandee->type=lecture_fichier;
            strcpy(action_demandee->filename,buffer_filename);
            return ;
        }
    }

    //lecture historique dans un fichier
    else if(strlen(ligne_commande)>=6 && strncmp(ligne_commande,"lit_h ",6)==0)
    {
        char buffer_filename[MAX_NOM_FICHIER_TAILLE];
        if(sscanf(ligne_commande,"lit_h %s",buffer_filename)==1)
        {
            printf("[debug Parseur]: parsing chargement (historique) %s\n",buffer_filename);fflush(stdout);

            action_demandee->type=lecture_historique;
            strcpy(action_demandee->filename,buffer_filename);
            return ;
        }
        else
        {
            printf("[debug Parseur]: Lecture historique non comprise\n");fflush(stdout);

            return ;
        }
    }

    printf("[debug Parseur]: ligne non valide \n");fflush(stdout);

}

int parse_ligne_commande_promotion(const char* ligne_commande,type_deplacement* coup)
{

    assert(coup!=NULL);

    if(ligne_commande==NULL)
        return 0;

    if(strlen(ligne_commande)<2)
        return 0;

    switch (ligne_commande[0])
    {
    case 'd':
        *coup=deplacement_promotion_dame;
        return 1;
    case 'f':
        *coup=deplacement_promotion_fou;
        return 1;
    case 't':
        *coup=deplacement_promotion_tour;
        return 1;
    case 'c':
        *coup=deplacement_promotion_cavalier;
        return 1;
    default:
        printf("Ligne de commande non reconnue, entrez 'd','f','t' ou 'c'\n");
        return 0;
    }

}
