
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 * Le parseur permet d'interagir avec le moteur du jeu d'echec a l'aide de la ligne de commande.
 * Le parseur est responsable de verifier l'integrite de la ligne de commande envoye par l'utilisateur.
 * Le parseur appel les commandes fournies par l'API_mode_interactif.
 * Il n'a pas connaissance des regles du jeu.
 *
 */

//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z


#ifndef PARSEUR_MODE_INTERACTIF_H
#define PARSEUR_MODE_INTERACTIF_H

#include "type_deplacement.h"

/**
 * **************************************************
 * **************************************************
 *
 * DECLARATION DES TYPES ET ENUMERATIONS DU PARSEUR
 *
 * **************************************************
 * **************************************************
 */


/** Les differents types d'actions possibles analysees par le parseur */
typedef enum{rien_a_faire,
             deplacement,
             lecture_fichier,
             lecture_historique,
             fin,
             initialisation}
            type_action;

/** Constante du nombre maximum de caratere de la ligne de commande */
#define MAX_LIGNE_COMMANDE 4096
/** Constante du nombre maximum de caratere du nom de fichier */
#define MAX_NOM_FICHIER_TAILLE 256

/** Conteneur pour stocker l'action a realiser ainsi que ses parametres */
typedef struct
{
    type_action type;                      //le type d'action a realiser

    char filename[MAX_NOM_FICHIER_TAILLE]; //stockage pour un nom de fichier a lire/ecrire

    int x_depart;                           //stockage pour une coordonnee de piece a deplacer
    int y_depart;                           //stockage pour une coordonnee de piece a deplacer
    int x_arrivee;                          //stockage pour une coordonnee de piece a deplacer
    int y_arrivee;                          //stockage pour une coordonnee de piece a deplacer

}action_a_realiser;


/**
 * **************************************************
 * **************************************************
 *
 * DECLARATION DES FONCTIONS DU PARSEUR
 *
 * **************************************************
 * **************************************************
 */

/**
 * Fonction action_initialise:
 * ********************************
 *   Initialise une action a sa valeur par defaut.
 *
 *   Necessite:
 *     - Un pointeur non constant vers une action. Pointeur non NULL.
 *   Garantie:
 *     - L'action contient un type "rien_a_faire", et le nom et les coordonnees sont initialisees a des valeurs par defaut.
 */
void action_initialise(action_a_realiser* action_a_initialiser);

/**
 * Fonction parse_ligne_commande:
 * ********************************
 *   Analyse une ligne entree en ligne de commande et la convertit en action correspondante.
 *   Le parseur n'analyse pas si l'action possede des parametres valides vis a vis du jeu d'echec.
 *
 *   Necessite:
 *     - Une chaine de Scaracteres provenant de la ligne de commande. Pointeur non NULL.
 *     - Un pointeur non constant vers une action_a_realiser mis a jour dans la fonction. Pointeur non NULL.
 *   Garantie:
 *     - Si la ligne de commande correspond a une syntaxe valide, l'action correspondante et ses parametres sont stockees dans la structure d'action.
 *     - Si la ligne de commande est invalide, l'action contient les valeurs par defauts.
 */
void parse_ligne_commande(const char* ligne_commande,action_a_realiser* action_demandee);

int parse_ligne_commande_promotion(const char* buffer_ligne_commande,type_deplacement* coup);

#endif
