
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 * Piece est un conteneur d'une piece d'un jeu.
 * Une piece contient un type (tour,cavalier,fou,dame,roi,pion-blanc/noir)
 *    ainsi que ses propres coordonnees.
 * Une piece n'a pas connaissance du comportement ou placement des autres pieces du jeu.
 * Les fonctions d'aides a la construction d'une piece d'un type donnee sont fournies
 *    par l'interface proposee par le fichier "constructeur_piece"
 *
 */

//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z



#ifndef PIECE_ECHIQUIER_H
#define PIECE_ECHIQUIER_H


/**
 * **************************************************
 * **************************************************
 *
 * DECLARATION DES TYPES ET ENUMERATIONS DE PIECE
 *
 * **************************************************
 * **************************************************
 */


/** Le type de piece est defini suivant sa valeur d'enum
 *
 *  Note: Un type de piece est considere comme "valide" si elle vaut:
 *                 tour,cavalier,fou,dame,roi,pion_blanc, ou pion_noir.
 */
typedef enum {tour=0,
              cavalier=1,
              fou=2,
              dame=3,
              roi=4,
              pion_blanc=5,
              pion_noir=6,
              invalide=7}
             type_de_piece;



/** La struct de type piece contenant les parametres d'une piece d'un jeu d'echec
    Une piece valide possede:
     - un type valide (compris entre 0 et 6)
     - deux coordonnees valides (comprises entre 0 et 7)
     Une piece invalide possede:
     - un type valant 7
     - deux coordonnees valant -1 chacune
     Toute autre configuration correspond a une erreur de code.
*/
typedef struct
{
    type_de_piece type;     //le type de piece
    int coord_x;            //la coordonnee en x
    int coord_y;            //la coordonnee en y
}piece;





/**
 * **************************************************
 * **************************************************
 *
 * DECLARATION DES FONCTIONS DE PIECE
 *
 * **************************************************
 * **************************************************
 */




/**
 * Fonction est_coordonnee_valide:
 * *************************
 *    Verifie que deux coordonnees (x,y) sont valides sur un echiquier.
 *    C'est a dire que x et y sont deux entiers compris dans l'intervalle [0,7].
 *
 *    Necessite:
 *      - Deux coordonnees de type entieres
 *    Garantie:
 *      - Un retour valant 1 si les coordonnees sont valides.
 *      - Un retour valant 0 si les coordonnees ne sont pas valides.
 */
int coordonnee_est_valide(int coordonnee_x,int coordonnee_y);



/**
 * Fonction piece_est_valide:
 * *************************
 *    --Description et contrat a completer--
 */
// Ecrire ici la signature de "piece_est_valide"





/**
 * Fonction piece_invalider:
 * *************************
 *    Invalide une piece donnee en parametre.
 *
 *    Une piece invalide possede un type valant "invalide" et
 *      l'ensemble des autres champs a -1.
 *
 *   Necessite:
 *      - Un pointeur non constant vers une piece valide. Pointeur non NULL.
 *   Garantie:
 *      - La piece pointee est invalide.
 *      - Les coordonnees de la piece pointee sont invalides.
 *      - Le type de la piece pointee est invalide.
 */
void piece_invalider(piece* piece_du_jeu);


/**
 * Fonction nommer_type_de_piece:
 * *************************
 *
 *  Fonction renvoyant la chaine de caractere correspondant au type de piece
 *  Utilisation comme une table de correspondance
 *
 *  Utilisation: La ligne suivante affichera "cavalier" a l'ecran:
 *   printf("%s\n",nommer_type_de_piece(cavalier));
 *
 *  Necessite:
 *      - Le type de piece (un entier positif)
 *   Garantie:
 *      - Le retour sous forme de chaine de caractere du type de piece.
 *        Retourne "erreur piece non valide" si le type n'existe pas.
 */
const char* nommer_type_de_piece(type_de_piece type);


/**
 * Fonction piece_affiche_debug:
 * *************************
 *    Affiche sur la sortie standard l'ensemble des champs d'une piece.
 *    Utile pour des fins de debug.
 *
 *   Necessite:
 *      - Un pointeur constant non NULL vers une piece du jeu valide ou non.
 *   Garantie:
 *      - L'affichage de la valeur des champs de la piece du jeu.
 */
void piece_afficher_debug(const piece* piece_du_jeu);



#endif
