
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 * Un type de deplacement encode le type de coup realise.
 * Il permet de distinguer les coups:
 *  - standards: une seule piece se deplace suivant les regles de bases.
 *  - speciaux: roque, prise en passant, et promotion.
 *
 * Le type de deplacement est utilise pour la gestion des coups de l'echiquier.
 *
 */


//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z


#ifndef TYPE_DEPLACEMENT
#define TYPE_DEPLACEMENT



/** Le type de deplacement encode le type de coup. */
typedef enum {deplacement_standard,         //coup standard (1 piece, un seul deplacement)
              deplacement_prise_en_passant,
              deplacement_roque,
              deplacement_promotion,          //indique qu'un pion doit etre promu (sans precision)
              deplacement_promotion_dame,     //promotion du pion en dame
              deplacement_promotion_tour,     //promotion du pion en tour
              deplacement_promotion_fou,      //promotion du pion en fou
              deplacement_promotion_cavalier, //promotion du pion en cavalier
              deplacement_invalide} type_deplacement; //deplacement non valide

/**
 * Fonction type_deplacement_est_valide:
 * ********************************
 *   Verifie si un type de deplacement est valide
 *
 *   Necessite:
 *     - Un type de deplacement en entree (entier non signe < 9).
 *   Garantie:
 *     - Renvoie 1 si le deplacement est valide.
 *     - Renvoie 0 si le deplacement est "deplacement_invalide"
 *
 */
int type_deplacement_est_valide(type_deplacement type);

#endif
