 
/********************************************************/
/* Chargement fichier off                               */
/********************************************************/

/* inclusion des fichiers d'en-tete Glut */
#include <math.h>
#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include <vector>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/stat.h>

const char *fichier_a_lire="../bin/echec_etat.txt";

/* Notre structure point */
typedef struct
{
  double x[3];
} sommet;
typedef struct
{
  int index[3];
} index_triangle;
typedef struct
{
  std::vector <sommet> geometry;
  std::vector <index_triangle> connectivity;
} mesh;

void mesh_scale(mesh *m,double s);

//void* analyse_texte(void *data);

mesh maillage;
mesh pion_mesh;
mesh tour_mesh;
mesh cavalier_mesh;
mesh fou_mesh;
mesh reine_mesh;
mesh roi_mesh;

mesh mesh_0,mesh_1,mesh_2,mesh_3,mesh_4,mesh_5,mesh_6,mesh_7,mesh_8,mesh_9;


char presse,presse_d=0;
int anglex=0,angley=45,translation_x=0,translation_y=0;
int camera_xold,camera_yold;
double zoom=-15.0;

/* Prototype des fonctions */
void affichage();
void clavier(unsigned char touche,int x,int y);
void reshape(int x,int y);
void idle(int );
void mouse(int bouton,int etat,int x,int y);
void mousemotion(int x,int y);
void draw_echiquier();
mesh load_off_file(const char* filename);


void draw_pion(float x,float y);
void draw_tour(float x,float y);
void draw_off(const mesh& mesh_to_draw);
void draw_cavalier(float x,float y);
void draw_fou(float x,float y);
void draw_reine(float x,float y);
void draw_roi(float x,float y);

struct piece
{
  int type;
  int px;
  int py;
};
#define MAX_PIECE 16
struct jeu
{
  struct piece donnee[MAX_PIECE];
};

jeu jeu_joueur_1;
jeu jeu_joueur_2;
int jeu_nombre_piece(const struct jeu *j);

void jeu_init(struct jeu* j);
void echiquier_lit(struct jeu *j1,struct jeu *j2,const char *fichier,int *joueur_a_jouer);
void echiquier_ecrit(const struct jeu *j1,const struct jeu *j2,const char *fichier);
int piece_est_valide(const struct piece *p);
void affiche_jeu(jeu* jeu_joueur);

time_t derniere_modification_fichier=0;
int joueur_a_jouer=1;
void affiche_coord();

int jeu_nombre_piece(const struct jeu *j)
{

  int counter=0;
  for(counter=0;counter<MAX_PIECE && piece_est_valide(&(j->donnee[counter]))==1;++counter);
  return counter;
}

int piece_est_valide(const struct piece *p)
{
  if(p->px>=0 && p->py>=0 && p->type>=0 && p->type<=7)
    return 1;
  
  return 0;
}

void echiquier_ecrit(const struct jeu *j1,const struct jeu *j2,const char *fichier)
{
  FILE *fid=NULL;
  fid=fopen(fichier,"w");
  if(fid==NULL)
    {
      printf("Erreur ouverture fichier %s\n",fichier);
      exit(1);
    }

  fprintf(fid,"Joueur 1\n");
  int N1=jeu_nombre_piece(j1);
  int k=0;
  for(k=0;k<N1;++k)
    {
      const struct piece *p=&j1->donnee[k];
      fprintf(fid,"%d %d %d\n",p->type,p->px,p->py);
    }

  fprintf(fid,"Joueur 2\n");
  int N2=jeu_nombre_piece(j2);
  for(k=0;k<N2;++k)
    {
      const struct piece *p=&j2->donnee[k];
      fprintf(fid,"%d %d %d\n",p->type,p->px,p->py);
    }


  fclose(fid);
}

void echiquier_lit(struct jeu *j1,struct jeu *j2,const char *fichier,int *joueur)
{
  jeu_init(j1);
  jeu_init(j2);

    FILE *fid=NULL;
    fid=fopen(fichier,"r");

    if(fid==NULL)
    {
        printf("Erreur ouverture fichier %s\n",fichier);
        exit(1);
    }

    int indice_j1=0;
    int indice_j2=0;

    int joueur_courant=0;
    char buffer[512];
    while(fgets(buffer,512,fid)!=0)
    {
      //printf("%s",buffer);
        if(strlen(buffer)>=9 && strncmp(buffer,"Joueur 1",8)==0)
        {
            joueur_courant=1;
            if(strlen(buffer)>=13 && strncmp(buffer+9,"(*)",3)==0)
                *joueur=1;
        }
        if(strlen(buffer)>=9 && strncmp(buffer,"Joueur 2",8)==0)
        {
            joueur_courant=2;
            if(strlen(buffer)>=13 && strncmp(buffer+9,"(*)",3)==0)
                *joueur=2;
        }


        int type=-1;
        int x=-1;
        int y=-1;
        if(sscanf(buffer,"%d %d %d",&type,&x,&y)==3)
        {
            if(joueur_courant==1)
            {
                j1->donnee[indice_j1].type=type;
                j1->donnee[indice_j1].px=x;
                j1->donnee[indice_j1].py=y;
                ++indice_j1;

            }
            else if(joueur_courant==2)
            {
                j2->donnee[indice_j2].type=type;
                j2->donnee[indice_j2].px=x;
                j2->donnee[indice_j2].py=y;
                ++indice_j2;
            }
        }

    }


    fclose(fid);
}


int main(int argc,char **argv)
{

  if(argc>=2)
    {
      fichier_a_lire=argv[1];
    }
  printf("Visualiseur lit fichier %s\n",fichier_a_lire);

  /* initialisation de glut et creation
     de la fenetre */
  glutInit(&argc,argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowPosition(0,0);
  glutInitWindowSize(550,550);
  glutCreateWindow("Test");

  /* Initialisation d'OpenGL */
  glClearColor(1.0,1.0,1.0,0.0);
  glColor3f(1.0,1.0,1.0);
  glPointSize(2.0);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_COLOR_MATERIAL);

  /* enregistrement des fonctions de rappel */
  glutDisplayFunc(affichage);
  glutKeyboardFunc(clavier);
  glutReshapeFunc(reshape);
  glutMouseFunc(mouse);
  glutMotionFunc(mousemotion);
  glutTimerFunc(100,idle,0);

  //light
  float light_ambient[4]={0.4,0.4,0.4,1.0};
  float light_diffuse[4]={0.4,0.4,0.4,1.0};
  float light_specular[4]={0.3,0.3,0.3,1.0};
  float light_position [4] = {2.0,100.0,100.0,1.0};

  float material_ambient [4] = {1.0,1.0,1.0,1.0};
  float material_diffuse [4] = {1.0,1.0,1.0,1.0};
  float material_specular [4] = {1.0,1.0,1.0,1.0};
  float material_exponent [1] = {60.0};


  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
  glLightfv(GL_LIGHT0, GL_POSITION, light_position);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, material_ambient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_diffuse);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, material_specular);  
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, material_exponent);
  glEnable(GL_DEPTH_TEST);


  //echiquier_init(&jeu_joueur_1,&jeu_joueur_2);
  echiquier_lit(&jeu_joueur_1,&jeu_joueur_2,fichier_a_lire,&joueur_a_jouer);

  

  pion_mesh=load_off_file("mesh/pion.off");
  tour_mesh=load_off_file("mesh/tour.off");
  cavalier_mesh=load_off_file("mesh/cavalier.off");
  fou_mesh=load_off_file("mesh/fou.off");
  reine_mesh=load_off_file("mesh/dame.off");
  roi_mesh=load_off_file("mesh/roi.off");

  mesh_0=load_off_file("mesh/text/0.off"); mesh_scale(&mesh_0,0.6);
  mesh_1=load_off_file("mesh/text/1.off"); mesh_scale(&mesh_1,0.6);
  mesh_2=load_off_file("mesh/text/2.off"); mesh_scale(&mesh_2,0.6); 
  mesh_3=load_off_file("mesh/text/3.off"); mesh_scale(&mesh_3,0.6);
  mesh_4=load_off_file("mesh/text/4.off"); mesh_scale(&mesh_4,0.6);
  mesh_5=load_off_file("mesh/text/5.off"); mesh_scale(&mesh_5,0.6);
  mesh_6=load_off_file("mesh/text/6.off"); mesh_scale(&mesh_6,0.6);
  mesh_7=load_off_file("mesh/text/7.off"); mesh_scale(&mesh_7,0.6);
  mesh_8=load_off_file("mesh/text/8.off"); mesh_scale(&mesh_8,0.6);
  mesh_9=load_off_file("mesh/text/9.off"); mesh_scale(&mesh_9,0.6);

  /* Charge fichier off */
  //load_off_file("venus.off");

  //  pthread_t thread_analyseur;
  //  pthread_create(&thread_analyseur,0,analyse_texte,0);

  /* Entree dans la boucle principale glut */
  glutMainLoop();
  return 0;
}

mesh load_off_file(const char* filename)
{

  mesh current_mesh;

#define SIZE_BUFFER 2048
  char buffer[SIZE_BUFFER]={'\0'};
  
  
  FILE *fid=NULL;
  fid=fopen(filename,"r");
  if(fid==NULL)
    {printf("ERROR loading %s in load_off_file\n",filename);exit(-1);}


  while(strcmp(buffer,"OFF")!=0)
    {
      if(fscanf(fid,"%s",buffer)==EOF)
	{printf("ERROR file %s is not off file\n",filename);exit(-1);}
      if(buffer[0]=='#')//comments
	fgets(buffer,sizeof(buffer),fid);
    }


  //pass comments
  fscanf(fid,"%s",buffer);
  while(buffer[0]=='#')//comments
    {
      fgets(buffer,sizeof(buffer),fid);
      fscanf(fid,"%s",buffer);
    }


  //read Number of vertices
  int N_vertex=0,N_polygon=0;
  int k_vertex=0;
  N_vertex=atoi(buffer);//N_vertex
  fscanf(fid,"%d",&N_polygon);//N_polygon
  fgets(buffer,sizeof(buffer),fid);//read line


  //read vertices
  float X[3];
  sommet temp_sommet;
  while(k_vertex<N_vertex)
    {
      if(fscanf(fid,"%f %f %f",X,X+1,X+2)!=3)
	{
	  fscanf(fid,"%s",buffer);
	  if(buffer[0]!='#')
	    {printf("ERROR loading %s\n",filename);exit(-1);}
	  else//read line
	    fgets(buffer,sizeof(buffer),fid);
	}
      else
	{
	  for(int k=0;k<3;k++)
	    temp_sommet.x[k]=X[k];
	  current_mesh.geometry.push_back(temp_sommet);

	  k_vertex++;
	}
    }

  //read connectivity
  int size_poly=0;
  int k_polygon=0;
  int k_current_polygon=0;
  int temp=0;
  std::vector <int> v_poly;
  index_triangle temp_triangle;
  for(k_polygon=0;k_polygon<N_polygon;k_polygon++)
    {
      if(fscanf(fid,"%d",&size_poly)!=1)
	{
	  fscanf(fid,"%s",buffer);
	  if(buffer[0]!='#')
	    {printf("ERROR loading triangles in load_off_file %s\n",filename);exit(-1);}
	  else//read line
	    {
	      fgets(buffer,sizeof(buffer),fid);
	      k_polygon--;
	    }
	}
      else
	{
	  v_poly.resize(0);
	  for(k_current_polygon=0;k_current_polygon<size_poly;k_current_polygon++)
	    {
	      fscanf(fid,"%d",&temp);
	      v_poly.push_back(temp);
	    }

	  for(unsigned int k=0;k<v_poly.size()-2;k++)
	    {
	      temp_triangle.index[0]=v_poly[0];
	      temp_triangle.index[1]=v_poly[k+1];
	      temp_triangle.index[2]=v_poly[k+2];
	      current_mesh.connectivity.push_back(temp_triangle);
	    }


	}
    }

  printf("[Chargement fichier %s OK]\n",filename);

  return current_mesh;
}


void affichage()
{

  /* effacement de l'image avec la couleur de fond */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


  //Projection
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity ();
  gluPerspective (45, 1.0, 0.05,1000.0);

  
  //Modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0,0.0,zoom);
  glRotatef(-angley/6.0,1.0,0.0,0.0);
  glRotatef(-anglex/6.0,0.0,0.0,1.0);
  glTranslatef(-4.0,-4.0,0.0);



  float light_specular_echiquier[4]={0.1,0.1,0.1,1.0};
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular_echiquier);

  draw_echiquier();



  float light_specular_piece[4]={0.6,0.6,0.6,1.0};
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular_piece);


  glColor3f(0.8,0.8,0.8);
  affiche_jeu(&jeu_joueur_1);

  glColor3f(0.2,0.2,0.2);
  affiche_jeu(&jeu_joueur_2);


  float light_specular_text[4]={0.2,0.2,0.2,1.0};
  glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular_text);

  affiche_coord();



  // for(unsigned int kx=0;kx<8;++kx)
  //   draw_pion(kx,1);
  // draw_tour(0,0);
  // draw_tour(7,0);
  // draw_cavalier(1,0);draw_cavalier(6,0);
  // draw_fou(2,0);draw_fou(5,0);
  // draw_reine(3,0);draw_roi(4,0);

  // glColor3f(0.2,0.2,0.2);
  // for(unsigned int kx=0;kx<8;++kx)
  //   draw_pion(kx,6);
  // draw_tour(0,7);
  // draw_tour(7,7);
  // draw_cavalier(1,7);draw_cavalier(6,7);
  // draw_fou(2,7);draw_fou(5,7);
  // draw_reine(4,7);draw_roi(3,7);

  if(joueur_a_jouer==1)
    {
      glColor3f(0.8,0.8,0.8);
      draw_pion(4,-2);
    }
  else if(joueur_a_jouer==2)
    {
      glColor3f(0.2,0.2,0.2);
      draw_pion(4,8);
    }

  
  /* On echange les buffers */
  glutSwapBuffers();
}

void clavier(unsigned char touche,int ,int )
{
  switch (touche)
    {
    case 'p': /* affichage du carre plein */
      glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
      glutPostRedisplay();
      break;
    case 'f': /* affichage en mode fil de fer */
      glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
      glutPostRedisplay();
      break;
    case 's' : /* Affichage en mode sommets seuls */
      glPolygonMode(GL_FRONT_AND_BACK,GL_POINT);
      glutPostRedisplay();
      break;
    case 'd':
      glEnable(GL_DEPTH_TEST);
      glutPostRedisplay();
      break;
    case 'D':
      glDisable(GL_DEPTH_TEST);
      glutPostRedisplay();
      break;
    case 'q' : /*la touche 'q' permet de quitter le programme */
      exit(0);
      break;
    case 'l':
      echiquier_lit(&jeu_joueur_1,&jeu_joueur_2,fichier_a_lire,&joueur_a_jouer);
      glutPostRedisplay();
      break;
    default:
      break;
    }
}

void reshape(int x,int y)
{
  if (x<y)
    glViewport(0,(y-x)/2,x,x);
  else 
    glViewport((x-y)/2,0,y,y);
}

void mouse(int button, int state,int x,int y)
{
  /* si on appuie sur le bouton gauche */
  if(state == GLUT_DOWN) 
  {
    camera_xold = x; /* on sauvegarde la position de la souris */
    camera_yold=y;
  }
  if(button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    presse=1;
  if(button == GLUT_LEFT_BUTTON && state==GLUT_UP)
    presse=0;

  if(button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
    presse_d=1;
  if(button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
    presse_d=0;

}

void draw_echiquier()
{
  float e=0.2;
  glBegin(GL_QUADS);
  for(unsigned int kx=0;kx<8;++kx)
    {
      for(unsigned int ky=0;ky<8;++ky)
	{
	  if((kx+ky)%2==0)
	    glColor3f(0,0,0);
	  else
	    glColor3d(0.8,0.8,0.8);

	  glNormal3f(0,0,1);
	  glVertex3f(kx,ky,0.0);
	  glVertex3f(kx+1,ky,0.0);
	  glVertex3f(kx+1,ky+1,0.0);
	  glVertex3f(kx,ky+1,0.0);

	  glNormal3f(0,0,-1);
	  glVertex3f(kx,ky,-e);
	  glVertex3f(kx+1,ky,-e);
	  glVertex3f(kx+1,ky+1,-e);
	  glVertex3f(kx,ky+1,-e);

	  glNormal3f(0,-1,0);
	  glVertex3f(kx,ky,0.0);
	  glVertex3f(kx+1,ky,0.0);
	  glVertex3f(kx+1,ky,-e);
	  glVertex3f(kx,ky,-e);

	  glNormal3f(-1,0,0);
	  glVertex3f(kx,ky,0.0);
	  glVertex3f(kx,ky+1,0.0);
	  glVertex3f(kx,ky+1,-e);
	  glVertex3f(kx,ky,-e);

	  glNormal3f(0,1,0);
	  glVertex3f(kx,ky+1,0.0);
	  glVertex3f(kx+1,ky+1,0.0);
	  glVertex3f(kx+1,ky+1,-e);
	  glVertex3f(kx,ky+1,-e);

	  glNormal3f(1,0,0);
	  glVertex3f(kx+1,ky,0.0);
	  glVertex3f(kx+1,ky+1,0.0);
	  glVertex3f(kx+1,ky+1,-e);
	  glVertex3f(kx+1,ky,-e);
	}
    }
  glEnd();


}

void mousemotion(int x,int y)
{
  if (presse) /* si le bouton gauche est presse */
    {
      /* on modifie les angles de rotation de l'objet
	 en fonction de la position actuelle de la souris et de la derniere
	 position sauvegardee */
      anglex=anglex-(x-camera_xold); 
      angley=angley-(y-camera_yold);
      glutPostRedisplay(); /* on demande un rafraichissement de l'affichage */
    }
  if(presse_d)
    {
      zoom = zoom + (y-camera_yold)/50.0;
      glutPostRedisplay(); /* on demande un rafraichissement de l'affichage */
    }

  
  camera_xold=x; /* sauvegarde des valeurs courante de le position de la souris */
  camera_yold=y;
}

 
void draw_pion(float x,float y)
{

  glPushMatrix();
  glTranslatef(x+0.5,y+0.5,0.5);

  draw_off(pion_mesh);

  glPopMatrix();
}
void draw_tour(float x,float y)
{

  glPushMatrix();
  glTranslatef(x+0.5,y+0.5,0.5);

  draw_off(tour_mesh);

  glPopMatrix();
}
void draw_cavalier(float x,float y)
{

  glPushMatrix();
  glTranslatef(x+0.5,y+0.5,0.5);

  draw_off(cavalier_mesh);

  glPopMatrix();
}
void draw_fou(float x,float y)
{

  glPushMatrix();
  glTranslatef(x+0.5,y+0.5,0.5);

  draw_off(fou_mesh);

  glPopMatrix();
}
void draw_reine(float x,float y)
{

  glPushMatrix();
  glTranslatef(x+0.5,y+0.5,0.5);

  draw_off(reine_mesh);

  glPopMatrix();
}
void draw_roi(float x,float y)
{

  glPushMatrix();
  glTranslatef(x+0.5,y+0.5,0.5);

  draw_off(roi_mesh);

  glPopMatrix();
}

void draw_off(const mesh& mesh_to_draw)
{
  int i=0;

  glPushMatrix();
  glTranslatef(translation_x,translation_y,0.0);

  int N=mesh_to_draw.connectivity.size();
  glBegin(GL_TRIANGLES);

  double X0[3],X1[3],X2[3];
  double u[3],v[3];
  double normal[3],n;
  for(i=0;i<N;i++)
    {
      for(int k=0;k<3;k++)
	{
	  X0[k] = mesh_to_draw.geometry[mesh_to_draw.connectivity[i].index[0]].x[k];
	  X1[k] = mesh_to_draw.geometry[mesh_to_draw.connectivity[i].index[1]].x[k];
	  X2[k] = mesh_to_draw.geometry[mesh_to_draw.connectivity[i].index[2]].x[k];
	}
      for(int k=0;k<3;u[k]=X1[k]-X0[k],v[k]=X2[k]-X0[k],k++);
      
      //color
      //      glColor3d(0.8,0.8,0.8);

      //normal
      normal[0]=u[1]*v[2]-u[2]*v[1];
      normal[1]=u[2]*v[0]-u[0]*v[2];
      normal[2]=u[0]*v[1]-u[1]*v[0];
      n = powf(normal[0]*normal[0]+normal[1]*normal[1]+normal[2]*normal[2],0.5);
      if(n>=0.0001)
	for(int k=0;k<3;k++)
	  normal[k]/=n;
      glNormal3dv(normal);

      //vertex
      glVertex3dv(X0);
      glVertex3dv(X1);
      glVertex3dv(X2);

    }

  glEnd();

  glPopMatrix();
}
	
	
void affiche_jeu(jeu* jeu_joueur)
{
  int N_piece_joueur=jeu_nombre_piece(jeu_joueur);
  for(int k=0;k<N_piece_joueur;++k)
    {
      piece *p=&jeu_joueur->donnee[k];
      switch(p->type)
      	{
      	case 5:
	case 6:
      	  draw_pion(p->px,p->py);
      	  break;
      	case 0:
      	  draw_tour(p->px,p->py);
      	  break;
      	case 1:
      	  draw_cavalier(p->px,p->py);
      	  break;
      	case 2:
      	  draw_fou(p->px,p->py);
      	  break;
      	case 3:
      	  draw_reine(p->px,p->py);
      	  break;
      	case 4:
      	  draw_roi(p->px,p->py);
      	  break;
	default:
	  printf("Attention type %d inconnu\n",p->type);
	  break;
      	}
    }


}

// void* analyse_texte(void *data)
// {
//   sleep(1);
//   char buffer[512];
  
//   int fin_entree=0;

//   while(fin_entree==0)
//     {
//       printf(">> ");
//       fflush(stdout);
//       fgets(buffer,sizeof(buffer),stdin);
  
//       printf("visualiseur: %s",buffer);
//       fflush(stdout);

//       echiquier_lit(&jeu_joueur_1,&jeu_joueur_2,"../projet/save.txt");
//       glutPostRedisplay();

      
//       if(strncmp(buffer,"FIN",3)==0)
// 	fin_entree=1;
//     }

// }

void idle(int )
{
  glutPostRedisplay();
  glutTimerFunc(300,idle,0);

  struct stat stat_buffer;
  int stat_return;
  stat_return=stat(fichier_a_lire,&stat_buffer);

  if(stat_return!=0)
    printf("Erreur stat fichier\n");

  if(derniere_modification_fichier!=stat_buffer.st_mtime)
    {
      derniere_modification_fichier=stat_buffer.st_mtime;
      echiquier_lit(&jeu_joueur_1,&jeu_joueur_2,fichier_a_lire,&joueur_a_jouer);
      glutPostRedisplay();
    }

  
}


void affiche_coord()
{
  glPushMatrix();
  glRotatef(180,1,0,0);
  glTranslatef(0,1.0,0.0);
  glTranslatef(0.5,-0.5,0.0);
  draw_off(mesh_0);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_1);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_2);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_3);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_4);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_5);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_6);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_7);
  glPopMatrix();

  glPushMatrix();
  glRotatef(180,1,0,0);
  glRotatef(-90,0,0,1);
  glTranslatef(0,0.0,0.0);
  glTranslatef(0.5,-0.5,0.0);
  draw_off(mesh_0);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_1);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_2);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_3);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_4);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_5);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_6);
  glTranslatef(1.0,0.0,0.0);
  draw_off(mesh_7);
  glPopMatrix();
}

void mesh_scale(mesh *m,double s)
{
  unsigned int N=m->geometry.size();
  for(unsigned int k=0;k<N;++k)
    {
      for(unsigned int k_dim=0;k_dim<3;++k_dim)
	m->geometry[k].x[k_dim]*=s;
    }
}

void jeu_init(struct jeu* j)
{
  for(unsigned int k=0;k<16;++k)
    {
      j->donnee[k].type=7;
      j->donnee[k].px=-1;
      j->donnee[k].py=-1;
    }
}
